(*
	Cutesy applet designed for inclusion in the Sorcerer Sampler pack.
	Doesn't do much unless you happen to have the out-of-memory problem
	with HCP.EXE, but useful as a template for ultra-compact Delphi applets.
	Shows what Delphi can do in under 8k.

	Resource Workshop hint: to get a little extra filesize optimization in
	all your Borland 16 bit apps, load them into Resource Workshop and save
	them after your final compile. You'll often get a filesize reduction
	of 1 to 10 percent when reswks relinks the executable.
*)



program ramcrush;
uses WinTypes,WinProcs,Messages;
{$R crusher.res}

{CTL3D variables}
var hiCtl3d      : THandle;             { Handle of CTL3D.DLL }
	ErrorMode    : Word;
	ThreeDReg, ThreeDAuto, ThreeDUnreg : pointer;
	Exitsave: Pointer;


{Generic error message}
procedure Showerr;
begin
	MessageBox(0,'RAMCrusher must be passed a numerical value between 2 and 256 '
		+' representing the number of megabytes of memory to be compacted. '
		+'For example:'+#10#10
		+'RAMCRUSH.EXE 16'+#10#10
		+'...will compact 16 megabytes of memory.'
		,'Invalid parameter detected',48);
end;

{Generic message for all compaction functions}
procedure ShowMsg;
begin
	MessageBox(GetActiveWindow,'If recompiling fails, restart Windows to '
		+'restore contiguous memory blocks.'
		+#10#10 +'This utility cannot infinitely reclaim VCPI memory. '
		+'If HCP.EXE "leaks" memory on your system, eventually you will '
		+'have to restart Windows to complete additional compiles.'
		,'Memory compacted...contiguous blocks freed',64);
end;

(**************Splash dialog funcs *********************)
function About(Dialog: HWnd; Message, WParam: Word;
  LParam: Longint): Longint; export;
var	Currect: TRect;
	cx,cy,x,y: integer;
begin
	About := 1;
	case Message of
		wm_initdialog: begin
{Get the dialog dimensions and center the window}
			GetWindowRect(Dialog,CurRect);
			cx := CurRect.Right - CurRect.Left;
			cy := CurRect.Bottom - CurRect.Top;
			x := (GetSystemMetrics(SM_CXSCREEN) - cx) div 2;
			y := (GetSystemMetrics(SM_CYSCREEN) - cy) div 2;
			SetWindowPos(Dialog,0,
				x,
				y,
				cx,cy,SWP_NOSIZE + SWP_SHOWWINDOW);
		end;
{process buttons}
		wm_Command: begin
			if (WParam = id_Cancel) then begin
				EndDialog(Dialog, 1);
				Exit;
			end;
			If (WParam = 1002) then begin
				GlobalCompact(2000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1004) then begin
				GlobalCompact(4000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1006) then begin
				GlobalCompact(6000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1008) then begin
				GlobalCompact(8000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1012) then begin
				GlobalCompact(12000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1016) then begin
				GlobalCompact(16000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1024) then begin
				GlobalCompact(24000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1032) then begin
				GlobalCompact(32000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
			If (WParam = 1048) then begin
				GlobalCompact(48000000);
				EndDialog(Dialog,1);
				Showmsg;
				Exit;
			end;
		end;
	end;
	About := 0;
end;


function WindowProc(Window:HWnd;Message,WPram:Word;LPram:Longint):Longint;export;
var Splashproc: TFarProc;
begin
  WindowProc := 0;
  case Message of
	wm_Create: begin
{This loads the dialog on creation of the window}
		SplashProc := MakeProcInstance(@About, HInstance);
		DialogBox(HInstance, 'AboutBox', Window, SplashProc);
		FreeProcInstance(SplashProc);
		Exit;
	end;
	wm_Destroy: begin
		PostQuitMessage(0);
		Exit;
	end;
  end;
  WindowProc := DefWindowProc(Window, Message, WPram, LPram);
end;

procedure WinMain;
var
  Window: HWnd;
  Message: TMsg;
const
  WindowClass: TWndClass = (
	style: 0;
	lpfnWndProc: @WindowProc;
	cbClsExtra: 0;
	cbWndExtra: 0;
	hInstance: 0;
	hIcon: 0;
	hCursor: 0;
	hbrBackground: 0;
	lpszMenuName: nil;
	lpszClassName: 'RAMCrusher');
begin
{ Initialise the CTL3D unit: if CTL3D available and not loaded, load it.
 It is unloaded from the Quit procedure in Winmain.}
	ErrorMode := SetErrorMode(SEM_NOOPENFILEERRORBOX);
	hiCtl3d := LoadLibrary( 'CTL3DV2.DLL' );
	if (hiCtl3d < HINSTANCE_ERROR) then hiCtl3d := LoadLibrary( 'CTL3D.DLL' );
	if (hiCtl3d >= HINSTANCE_ERROR) then Begin
		ThreeDReg  := GetProcAddress( hiCtl3d, 'Ctl3dRegister'     );
		ThreeDAuto := GetProcAddress( hiCtl3d, 'Ctl3dAutoSubclass' );
		ThreeDUnReg:= GetProcAddress( hiCtl3d, 'Ctl3dUnRegister'   );
		if (ThreeDReg <> NIL) and (ThreeDAuto <> NIL) and (ThreeDUnReg <> NIL) then begin
			asm	Push HInstance
				call ThreeDReg
				Push HInstance
				Call ThreeDAuto
			End;
		End;
	End;{ctl3d init}
	SetErrorMode( ErrorMode ); {restore normal errormode...now be sure ctl3d is in exitproc}

  if HPrevInst = 0 then
  begin
	WindowClass.hInstance := HInstance;
	WindowClass.hCursor := LoadCursor(0, idc_Arrow);
	WindowClass.hbrBackground := GetStockObject(white_Brush);
	if not RegisterClass(WindowClass) then Halt(255);
  end;
  Window := CreateWindow(
	'RAMCrusher',
	'RAMCrusher',
	ws_OverlappedWindow,
	cw_UseDefault,
	cw_UseDefault,
	cw_UseDefault,
	cw_UseDefault,
	0,
	0,
	HInstance,
	nil);
{Hiding the main window makes this a dialog-only applet;
 this seems easier than tracking down a copy of AppStudio
 just to do a tiny applet when Resource Workshop is probably
 already on your system.}
{+++}  ShowWindow(Window,SW_HIDE);
{  Halt(Message.wParam);}
end;

procedure Quit;
begin
	Exitproc := Exitsave;
	if (hiCtl3d >= HINSTANCE_ERROR) then begin
		asm Push HInstance
			call ThreeDUnReg
		end;
		FreeLibrary(hiCtl3d);
	end;
end;


var l1: longint;
	Errcode: Integer;
begin
	Exitsave := Exitproc;
	Exitproc := @Quit;
	If paramcount = 0 then Winmain;
	If paramcount = 1 then begin
		Val(paramstr(1),l1,Errcode);
		If (Errcode <> 0) or (l1 > 256) then Showerr else begin
			GlobalCompact(l1*1000000);
			ShowMsg;
		End;
	end;
	If Paramcount > 1 then ShowErr;
end.


